/*------------------------------------------------------------------------
  Find the value of the kth smallest element in array A[n] by partial
  partitioning sort.  The values in the matrix are shuffled and
  partially ordered, keeping at each iteration a live interval [lo,hi]
  that is guaranteed to contain the kth element.
  ------------------------------------------------------------------------*/

#ifndef ELEM
# define ELEM double
#endif
#define ESWAP(a,b) {ELEM __t=(a); (a)=(b); (b)=__t;}

ELEM 
dpercentile(unsigned long k, ELEM A[], unsigned long n)
{
  unsigned long lo=0, hi=n-1;
  while (hi > lo+1) {
    /* Find cut value b with a<b<c from among A[lo],A[mid],A[hi].
     */
    unsigned long mid = (lo+hi) >> 1, l=lo+1, h=hi;
    ELEM a=A[lo], b=A[mid], c=A[hi];
    if (b>c) { ESWAP(b,c); }
    if (a>c) { ESWAP(a,c); }
    if (a>b) { ESWAP(a,b); }
    A[mid] = A[l];
    A[l] = a;
    A[hi] = c;
    for (;;) {
      do l++; while (A[l] < b);
      do h--; while (A[h] > b);
      if (h < l) break;
      ESWAP(A[l],A[h]);
    }
    A[lo] = A[h];
    A[h] = b;
    if (h >= k) hi = h-1;
    if (h <= k) lo = l;
  }
  if (hi>lo && A[hi]<A[lo]) { ESWAP(A[lo],A[hi]); }
  return A[k];
}
#undef ESWAP
